//-------------------------------------------------------------------------------------------------------------------------------
{xml Headers and Footers}
function kml_Header1: String;
begin
  Result := '<?xml version="1.0" encoding="UTF-8"?>' + CRLF +
            '<kml xmlns="http://earth.google.com/kml/2.2">' + CRLF +
            '  <Document>' + CRLF +
            '    <name><![CDATA[aDrake]]></name>' + CRLF +
            '    <open>1</open><visibility>1</visibility>' + CRLF;
end;           
function kml_Header2(Name, Color: String): String;
begin
  Result := '    <Placemark>' + CRLF +
            '      <Snippet></Snippet>' + CRLF +
            '      <Style><LineStyle><color>' + Color + '</color><width>4</width></LineStyle></Style>' + CRLF +
            '      <name><![CDATA[' + AnsiToUtf(Name) + ']]></name>' + CRLF +
            '      <description><![CDATA[Pocket Drake track]]></description>' + CRLF +
            '      <MultiGeometry>' + CRLF +
            '        <LineString>' + CRLF +
            '          <tessellate>1</tessellate>' + CRLF +
            '          <altitudeMode>clampToGround</altitudeMode>' + CRLF +
            '          <coordinates>' + CRLF;
end;           
function kml_Footer2(): String;
begin
  Result := '          </coordinates>' + CRLF +
            '        </LineString>' + CRLF +
            '      </MultiGeometry>' + CRLF +
            '    </Placemark>' + CRLF;
end;              
function kml_Footer1(): String;
begin
  Result := '  </Document>' + CRLF +
            '</kml>';
end;              
function plt_Header(Name: String): String;
begin
  Result := 'OziExplorer Track Point File Version 2.0' + CRLF +          //Line 1 : File type and version information
            'WGS 84' + CRLF +                                            //Line 2 : Geodetic Datum used for the Lat/Lon positions for each trackpoint
            'Altitude is in Feet' + CRLF +                               //Line 3 : "Altitude is in feet" - just a reminder that the altitude is always stored in feet
            'Reserved' + CRLF +                                          //Line 4 : Reserved for future use
            '0,2,255,' + AnsiToUtf(Name) + ',1' + CRLF +                 //Line 5 : multiple fields as below
                                                                                   {Field 1 : always zero (0)
                                                                                    Field 2 : width of track plot line on screen - 1 or 2 are usually the best
                                                                                    Field 3 : track color (RGB)
                                                                                    Field 4 : track description (no commas allowed)
                                                                                    Field 5 : track skip value - reduces number of track points plotted, usually set to 1 
                                                                                    Field 6 : track type - 0 = normal , 10 = closed polygon , 20 = Alarm Zone
                                                                                    Field 7 : track fill style - 0 =bsSolid; 1 =bsClear; 2 =bsBdiagonal; 3 =bsFdiagonal; 4 =bsCross; 
                                                                                                                 5 =bsDiagCross; 6 =bsHorizontal; 7 =bsVertical;
                                                                                    Field 8 : track fill color (RGB)  }
            '0' + CRLF;                                                  //Line 6 : Number of track points in the track, not used, the number of points is determined when reading the points file
end;           
function plt_Footer(): String;
begin
  Result := '';
end;              
function gpx_Header(Name: String): String;
begin
  Result := '<?xml version="1.0" encoding="UTF-8" standalone="no" ?>' + CRLF +
            '<gpx xmlns="http://www.topografix.com/GPX/1/1" creator="aDrakeSync" version="1.1">' + CRLF +
            '  <trk>' + CRLF +
            '    <name><![CDATA[' + AnsiToUtf(Name) + ']]></name>' + CRLF +
            '    <trkseg>' + CRLF;
end;           
function gpx_Footer(): String;
begin
  Result := '    </trkseg>' + CRLF +
            '  </trk>' + CRLF +
            '</gpx>';
end;              
//===============================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------
{Track Form Active/Inactive}
procedure form_TrkInh(Active: Boolean);
begin
  TrkForm_btn_Combine       .Enabled :=  Active;
  TrkForm_btn_Divide        .Enabled :=  Active;
  TrkForm_btn_Rename        .Enabled :=  Active;
  TrkForm_btn_New           .Enabled :=  Active;
  TrkForm_btn_Refresh       .Enabled :=  Active;
  TrkForm_btn_Upload        .Enabled :=  Active and not ADSForm_rb_Offline.Checked;
  TrkForm_btn_Download      .Enabled :=  Active and not ADSForm_rb_Offline.Checked;
  TrkForm_btn_Preview       .Enabled :=  Active;
  TrkForm_btn_Save          .Enabled :=  Active;
  TrkForm_btn_Delete        .Enabled :=  Active;
  TrkForm_btn_Trip          .Enabled :=  Active;
  TrkForm_btn_Close         .Enabled :=  Active;
  TrkForm_lb_Track          .Enabled :=  Active;
  TrkForm_ed_TrkName        .Enabled :=  Active;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Make DateTime from Track record}
function MakeTrkDT(Track: TSqliteTable): TDateTime;
var
  str : String;
begin
  str    := RegexExtract('\d{6}$', '000000' + Track.FieldAsString(Track.FieldIndex['time']));    // doplni na format hhmmss a orizne
  Result := IntToDate(Track.FieldAsString(Track.FieldIndex['date'])) + EncodeTime(StrToInt(copy(str,1,2)),StrToInt(copy(str,3,2)),StrToInt(copy(str,5,2)),0);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Get Track ID from string}
function GetTrackID(str: String): String;
begin
  Result := RegexExtract('\d+$', str);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Get Track Name from string}
function GetTrackName(str: String): String;
begin
  Result := RegexExtract('^[^'+TAB+']*', str);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Update num of track records}
procedure TrackNumUpd(db: TSqliteDatabase; ListID: TStringList);
var  
  i         : Integer;
  Track_num : Integer;
begin
  with TrkForm_lb_Track do begin
    for i := (ListID.Count - 1) downto 0 do begin
      Track_num := db.GetTableValue('SELECT count(*) FROM track_log WHERE trackid = ' + ListID[i]);
      db.ExecSQL('UPDATE track_h SET size = ' + IntToStr(Track_num) + ' WHERE trackid = ' + ListID[i]);     
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Track list selection restore}
procedure TrackSelRest(ListID : TStringList);
var  
  i,j : Integer;
begin
  with TrkForm_lb_Track do begin
    for i := (Items.Count - 1) downto 0 do begin
      for j := (ListID.Count - 1) downto 0 do if GetTrackID(Items[i]) = ListID[j] then Selected[i] := True;   //v novem seznamu oznaci vznikle spojene stopy
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - zobrazeni zakl. info o stope}
procedure form_TrkInfo(Sender: TObject);
begin
  with TrkForm_lb_Track do begin
    TrkForm_ed_TrkName.Text := GetTrackName(Items[ItemIndex]);
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Read Track up}
procedure form_TrkRead(Sender: TObject);
var  
  Track_db            : TSqliteDatabase; // databaze s tabulkou Stopy 
  Track               : TSqliteTable;    // tabulka bodu Stopy
  str                 : String;
  id, size            : String;
begin
  form_TrkInh(False);
  if (TBitBtn(Sender).Name <> '') then Trk_List_Arrng := TBitBtn(Sender).Name;
  Track_db := TSqliteDatabase.Create(MakroPath + 'Data\_adrake.db3');
  try
    with TrkForm_lb_Track do begin
      Sorted := False;
      Items.Clear;
      if SQLTableExists(Track_db, 'track_h') then begin
        case Trk_List_Arrng of       // nacte tabulku stop dle zvoleneho razeni
          'btn_ArrName_Dwn': Track := Track_db.GetTable('SELECT * FROM track_h ORDER BY name    ASC'            , false);
          'btn_ArrName_Up' : Track := Track_db.GetTable('SELECT * FROM track_h ORDER BY name    DESC'           , false);
          'btn_ArrDate_Dwn': Track := Track_db.GetTable('SELECT * FROM track_h ORDER BY date    ASC,  time ASC' , false);
          'btn_ArrDate_Up' : Track := Track_db.GetTable('SELECT * FROM track_h ORDER BY date    DESC, time DESC', false);
          'btn_ArrID_Dwn'  : Track := Track_db.GetTable('SELECT * FROM track_h ORDER BY trackid ASC'            , false);
          'btn_ArrID_Up'   : Track := Track_db.GetTable('SELECT * FROM track_h ORDER BY trackid DESC'           , false);
          else               Track := Track_db.GetTable('SELECT * FROM track_h ORDER BY date    DESC, time DESC', false);
        end;
        try
          while not Track.EOF do begin
            id   := Track.FieldAsString(Track.FieldIndex['trackid']);                        //id
            size := Track.FieldAsString(Track.FieldIndex['size']);                           //num
            str  := UtfToAnsi(Track.FieldAsString(Track.FieldIndex['name']));                //nazev stopy
            str  := FixLen(str,TrkForm_ed_TrkName.MaxLength+2) + FormatDateTime('dd.mm.yyyy hh:mm',  MakeTrkDT(Track));
            Items.Add( str + TAB + TAB + TAB + FixLen(size,7) + TAB + id);
            Track.next;                   // skok na dalsi
          end;
          if str = '' then Items.Add(_('- no track in table -'));
        except
          ShowErrorMessage(ExceptionToString(ExceptionType, ExceptionParam));
        finally
          Track.free;
        end;
      end
      else Items.Add('- track table not found -');
    end;
  except
    ShowErrorMessage(ExceptionToString(ExceptionType, ExceptionParam));
  finally
    Track_db.free;
  end;  
  form_TrkInh(True);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Divide Track}
procedure TrkDivide;
var  
  Track_db                 : TSqliteDatabase; // databaze s tabulkou Stopy 
  Track                    : TSqliteTable;    // tabulka bodu Stopy
  Track_num                : Integer;         // pocet zaznamu trasy v tabulce
  SelItems                 : TStringList;     // ktere stopy v seznamu maji byt na konci oznaceny
  lat, lon                 : extended;
  lat_last, lon_last, dist : extended;
  dt, dt_last              : TDateTime;
  id, key                  : String;
  id_new                   : Integer;
  cond2, cond3             : boolean;
  i,j,k,m                  : Integer;
begin
  form_TrkInh(False);
  Track_db := TSqliteDatabase.Create(MakroPath + 'Data\_adrake.db3');
  try
    with TrkForm_lb_Track do begin
      SelItems:=TStringList.Create;
      m := SelCount;  //pocet oznacenych stop v seznamu
      for i := (Items.Count - 1) downto 0 do begin 
        if Selected[i] then begin
          k := k + 1; j := 0; id_new := 0;
          GB_Progress(1,k,m);
          id := GetTrackID(Items[i]);
          SelItems.Add(id);    //prida puvodni oznacenou trasu
          if id <> '' then begin
            Track_num := Track_db.GetTableValue('SELECT size FROM track_h WHERE trackid = ' + id);
            if Track_num > 0 then begin
              Track     := Track_db.GetTable     ('SELECT *        FROM track_log WHERE trackid = ' + id + ' ORDER BY date ASC, time ASC', false);
              try
                dt_last  := MakeTrkDT(Track);
                lat_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lat']));
                lon_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lon']));
                Track_db.ExecSQL('BEGIN TRANSACTION');
                while not Track.EOF do begin
                  if GB_Test then Break;  //preruseno uzivatelem
                  j := j + 1;
                  GB_Progress(2,j,Track_num);
                  key  := Track.FieldAsString(Track.FieldIndex['key']);
                  dt   := MakeTrkDT(Track)
                  lat  := StrToFl(Track.FieldAsString(Track.FieldIndex['lat']));
                  lon  := StrToFl(Track.FieldAsString(Track.FieldIndex['lon']));
                  dist := PointDistance(lat_last, lon_last, lat, lon);
                  cond2 := (abs(dt - dt_last) < TrkDivideForm_dt_TimeToTrim.Time) or not TrkDivideForm_cb_TimeToTrimEnbl.Checked;                  // deleni dle casovych rozestupu mezi dvema body
                  cond3 := (dist < StrToFl(TrkDivideForm_ed_DistToTrim.Text))     or not TrkDivideForm_cb_TrackToTrimEnbl.Checked;                 // deleni dle vzdalenosti mezi dvema body
                  if cond2 and cond3 then begin
                    if  id_new <> 0 then Track_db.ExecSQL('UPDATE track_log SET trackid = ' + IntToStr(id_new) + ' WHERE key = ' + key);  //oznaci vzorky novym trackid    
                  end
                  else begin
                    Track_db.ExecSQL('COMMIT');
                    Track_db.ExecSQL('INSERT INTO track_h (name,date,time,size) VALUES(' + '"' + 'track_' + FormatDateTime('yyyymmdd_hhmmss', dt) + '"' + ',' + FormatDateTime('yyyymmdd,hhmmss', dt)  + ',0)');      
                    id_new := Track_db.GetTableValue('SELECT trackid FROM track_h WHERE name = ' + '"' + FormatDateTime('yyyymmdd_hhmmss', dt) + '"');
                    SelItems.Add(IntToStr(id_new));   //prida novou stopu do seznamu oznacenych radku
                    Track_db.ExecSQL('UPDATE track_log SET trackid = ' + IntToStr(id_new) + ' WHERE key = ' + key);     
                    Track_db.ExecSQL('BEGIN TRANSACTION');
                  end;
                  dt_last  := dt;
                  lat_last := lat;
                  lon_last := lon;
                  Track.next;                   // skok na dalsi
                end;
                Track_db.ExecSQL('COMMIT');
              finally
                Track.free;
              end;
            end;
          end;
        end;
      end;
    end;
    TrackNumUpd(Track_db,SelItems);        //obnovi pocet zaznamu pro rozdelene stopy
  finally
    Track_db.free;
  end;  
  form_TrkRead(TrkForm_btn_Refresh);       //nacte aktualizovany seznam stop
  TrackSelRest(SelItems);                  //obnovi oznaceni polozek seznamu stop
  GB_Clean;
  GB_Progress(2,0,100);
  form_TrkInh(True);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Combine Tracks}
procedure form_TrkCombine(Sender: TObject);
var
  Track_db     : TSqliteDatabase; // databaze s tabulkou Stopy 
  SelItems     : TStringList;     // ktere stopy v seznamu maji byt na konci oznaceny
  id, id_first : String;
  i, j         : Integer;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrkInh(False);
      SelItems:=TStringList.Create;
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\_adrake.db3');
      try
        for i := (Items.Count - 1) downto 0 do begin 
          GB_Progress(1,Items.Count-i,Items.Count);
          if Selected[i] then begin
            id := GetTrackID(Items[i]);
            if id <> '' then begin
              if j > 0 then begin
                Track_db.ExecSQL('DELETE FROM track_h WHERE trackid = "' + id + '"');
                Track_db.ExecSQL('UPDATE track_log SET trackid = ' + id_first + ' WHERE trackid = ' + id);      
              end
              else begin 
                id_first := id;      //timto ID se oznaci cela spojovana stopa
                SelItems.Add(id);    //prida spojenou stopu mezi oznacene v seznamu
              end;
              j := j + 1;
            end;
          end;
        end;
        TrackNumUpd(Track_db,SelItems);       //obnovi pocet zaznamu pro rozdelene stopy
      finally
        Track_db.free;
      end;
      form_TrkRead(TrkForm_btn_Refresh);      //nacte aktualizovany seznam stop
      TrackSelRest(SelItems); //obnovi oznaceni polozek seznamu stop
      GB_Clean;
      form_TrkInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Save Track}
function TrkSave(NameOfFile: String; SingleFile: Boolean): String;
var
  Track_db  : TSqliteDatabase; // databaze s tabulkou Stopy 
  Track     : TSqliteTable;    // tabulka bodu Stopy
  Track_num : Integer;         // pocet zaznamu trasy v tabulce
  xml       : String;
  FileName  : String;
  FileExt   : String;
  FilePath  : String;
  id, str   : String;
  i,j,k,m   : Integer;
  Color_num : String;
begin
  with TrkForm_lb_Track do begin
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\_adrake.db3');
      try
        FileExt    := RegexExtract('\.\w{3}$', NameOfFile);
        FilePath   := ReplaceString(NameOfFile, '\' + RegexExtract('[^\\]+\.\w{3}$', NameOfFile), '');
        m := SelCount;  //pocet oznacenych stop v seznamu
        for i := (Items.Count - 1) downto 0 do begin 
          if Selected[i] then begin
            GB_Progress(1,k+1,m);
            id := GetTrackID(Items[i]);
            if id <> '' then begin
              case (k mod 10) of
                0 :     Color_num := 'ff00ffff';     //zluta
                1 :     Color_num := 'ff00ff00';     //zelena
                2 :     Color_num := 'ffffff00';     //svetle modra
                3 :     Color_num := 'ffff0000';     //tmave modra
                4 :     Color_num := 'ffff00ff';     //fialova
                5 :     Color_num := 'ff0000ff';     //cervena
                6 :     Color_num := 'ff0067ff';     //oranzova
                7 :     Color_num := 'ffbababa';     //sediva
                8 :     Color_num := 'ffffffff';     //bila
                9 :     Color_num := 'ff000000';     //cerna
                else    Color_num := 'fff60d6b';     //zeleno sediva
              end;
              k := k + 1;
              //pokud je vybrano vice stop, pak nazvy souboru dle stop
              if m > 1 then FileName := SystemName(GetTrackName(Items[i]))   //nahrazeni nepovolenych (mimo uvedenych) znaku
                       else FileName := ReplaceString(RegexExtract('[^\\]+\.\w{3}$', NameOfFile), FileExt, '');
              if not SingleFile or (k = 1) then NameOfFile := FilePath + '\' + FileName + FileExt;
              Track_num  := Track_db.GetTableValue('SELECT size FROM track_h    WHERE trackid = "' + id + '"');
              Track      := Track_db.GetTable     ('SELECT *    FROM track_log  WHERE trackid = "' + id + '" ORDER BY date ASC, time ASC', false);
              try
                case FileExt of 
                  '.kml': begin
                            if not SingleFile or (k = 1) then xml := kml_Header1;
                            xml := xml + kml_Header2(FileName, UpperCase(Color_num));
                          end;
                  '.plt': xml := plt_Header(FileName);
                  '.gpx': xml := gpx_Header(FileName);
                end;
                j := 0;
                while not Track.EOF do begin
                  if GB_Test then Break;  //preruseno uzivatelem
                  j := j + 1;
                  GB_Progress(2,j,Track_num);
                  case FileExt of 
                    '.kml': xml := xml + '            ' + Track.FieldAsString(Track.FieldIndex['lon']) + ',' + Track.FieldAsString(Track.FieldIndex['lat']) + ',' + Track.FieldAsString(Track.FieldIndex['alt']) + CRLF;                      //lon, lat, alt
                    '.plt': xml := xml + Track.FieldAsString(Track.FieldIndex['lat']) + ',' + Track.FieldAsString(Track.FieldIndex['lon']) + ',0,' + FloatToStr(StrToFloat(Track.FieldAsString(Track.FieldIndex['alt'])) * 3.28084) + ',' +   //lat, lon, code, alt(in feet)
                                         FloatToStr(IntToDate(Track.FieldAsString(Track.FieldIndex['date'])) + (StrToFloat(Track.FieldAsString(Track.FieldIndex['time'])) / 235959)) + ',,' + CRLF;                       //DateTime, [StringDate], [StringTime]
                    '.gpx': begin
                             str := RegexExtract('\d{6}$', '000000' + Track.FieldAsString(Track.FieldIndex['time']));
                             insert(':',str,3); insert(':',str,6);
                             str := Track.FieldAsString(Track.FieldIndex['date']) + 'T' + str + 'Z';
                             insert('-',str,5); insert('-',str,8);
                             xml := xml + '     <trkpt lat="' + Track.FieldAsString(Track.FieldIndex['lat']) + '" lon="' + Track.FieldAsString(Track.FieldIndex['lon']) + '"><ele>' + Track.FieldAsString(Track.FieldIndex['alt']) + 
                                               '</ele><time>' + str  + '</time></trkpt>' + CRLF;
                           end;
                  end;
                  Track.next;
                end;
                case FileExt of 
                  '.kml': begin
                            xml := xml + kml_Footer2;
                            if not SingleFile then xml := xml + kml_Footer1;
                          end;
                  '.plt': xml := xml + plt_Footer;
                  '.gpx': xml := xml + gpx_Footer;
                end;
                if not SingleFile then begin
                  MakeDirPC(FilePath);
                  StringToFile(xml, NameOfFile);
                end;
              finally
                Track.free;
              end;
            end;
          end;
        end;
        if xml <> '' then begin
          Result := NameOfFile;
          if SingleFile then begin
            MakeDirPC(FilePath);
            xml := xml + kml_Footer1;
            StringToFile(xml, NameOfFile);
          end;
        end;
      finally
        Track_db.free;
      end;
    end; 
    GB_Clean;
    GB_Progress(2,0,100);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Preview Track by a kml asociated aplication}
procedure form_TrkView(Sender: TObject);
var
  file : String;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrkInh(False);
      file := TrkSave(MakroPath + 'Data\Preview.kml', True);
      if file <> '' then RunShell(file);
      form_TrkInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Save Track to File}
procedure form_TrkSave(Sender: TObject);
var
  file : String;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrkInh(False);
      file := SystemName(GetTrackName(Items[ItemIndex])) + '.kml';  
      if SaveDlg(_('Save aDrake track'), SmartPath(PC_TrackPath), '', _('GoogleEarth ') + '(*.kml)|*.kml|' + 
                                                                      _('OziExplorer ') + '(*.plt)|*.plt|' +
                                                                      _('GPX files ')   + '(*.gpx)|*.gpx', file) then begin
        if not RegexFind('((\.kml$)|(\.plt$))|(\.gpx$)', file) then file := file + '.kml';                                                                      
        TrkSave(file, False);
      end;
      form_TrkInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Delete Track}
procedure form_TrkDelete(Sender: TObject);
var
  Track_db  : TSqliteDatabase; // databaze s tabulkou Stopy 
  id        : String;
  i         : Integer;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrkInh(False);
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\_adrake.db3');
      try
        for i := (Items.Count - 1) downto 0 do begin 
          GB_Progress(1,(Items.Count-i),Items.Count);
          if Selected[i] then begin
            id := GetTrackID(Items[i]);
            if id <> '' then begin
              Track_db.ExecSQL('DELETE FROM track_log  WHERE trackid = "' + id + '"');
              Track_db.ExecSQL('DELETE FROM track_h    WHERE trackid = "' + id + '"');
              Items.Delete(i);
            end; 
          end;
        end;
        Track_db.ExecSQL('Vacuum');
      finally
        Track_db.free;
      end;
      GB_Clean;
      form_TrkInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Rename Track}
procedure form_TrkRename(Sender: TObject);
var
  Track_db    : TSqliteDatabase; // databaze s tabulkou Stopy 
  ii          : Integer;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) and (GetTrackID(Items[ItemIndex]) <> '') then begin       // pokud je vybrana nejaka stopa a je nactene ID
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\_adrake.db3');
      try
        Track_db.ExecSQL('UPDATE track_h SET name = "' + AnsiToUtf(TrkForm_ed_TrkName.Text) + '" WHERE trackid = ' + GetTrackID(Items[ItemIndex]));
      finally
        Track_db.free;
      end;
      ii := ItemIndex;
      form_TrkRead(TrkForm_btn_Refresh);    //nacte aktualizovany seznam stop
      Selected[ii] := True; //oznaci puvodni radek
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Add a new Track}
procedure form_TrkNew(Sender: TObject);
var
  Track_db    : TSqliteDatabase; // databaze s tabulkou Stopy 
begin
  Track_db := TSqliteDatabase.Create(MakroPath + 'Data\_adrake.db3');
  try
    Track_db.ExecSQL('INSERT INTO track_h (name,date,time,size) VALUES(' + '"' + AnsiToUtf(TrkForm_ed_TrkName.Text) + '"' + ',' + FormatDateTime('yyyymmdd,hhmmss', now) + ',0)');      
  finally
    Track_db.free;
  end;
  form_TrkRead(TrkForm_btn_Refresh);    //nacte aktualizovany seznam stop
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - zobrazeni Vyletu}
procedure form_TrkTrip(Sender: TObject);
var
  Track_db              : TSqliteDatabase; // databaze s tabulkou Stopy 
  Track                 : TSqliteTable;    // tabulka bodu Stopy
  Track_num             : Integer;         // pocet zaznamu trasy v tabulce
  id                    : String;
  lat, lon, alt         : extended;
  lat_last, lon_last    : extended;
  TripLen, dist         : extended;
  alt_min, alt_max      : extended;
  dt, dt_first, dt_last : TDateTime;
  i,j,k,m               : Integer; 
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrkInh(False);
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\_adrake.db3');
      try
        k := 0; alt_min := 9999;  //9.9999 km
        m := SelCount;  //pocet oznacenych stop v seznamu
        for i := (Items.Count - 1) downto 0 do begin 
          if Selected[i] then begin
            k := k + 1;
            GB_Progress(1,k,m);
            id := GetTrackID(Items[i]);
            if id <> '' then begin
              Track_num  := Track_db.GetTableValue('SELECT size FROM track_h    WHERE trackid = ' + id);
              Track      := Track_db.GetTable     ('SELECT *    FROM track_log  WHERE trackid = ' + id + ' ORDER BY date ASC, time ASC', false);
              if Track_num <> 0 then begin
                try
                  lat_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lat']));
                  lon_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lon']));
                  dt_first := MakeTrkDT(Track);
                  j := 0;
                  while not Track.EOF do begin
                    j := j + 1;
                    GB_Progress(2,j,Track_num);
                    lat := StrToFl(Track.FieldAsString(Track.FieldIndex['lat']));
                    lon := StrToFl(Track.FieldAsString(Track.FieldIndex['lon']));
                    alt := StrToFl(Track.FieldAsString(Track.FieldIndex['alt']));
                    //delka
                    dist    := PointDistance(lat_last, lon_last, lat, lon);
                    TripLen := TripLen + dist;
                    //cas
                    dt_last := MakeTrkDT(Track);
                    //vyska
                    if (alt_min > alt) and (alt <> 0) then alt_min := alt;
                    if (alt_max < alt)                then alt_max := alt;
                    //-----
                    lat_last := lat;
                    lon_last := lon;
                    Track.next;
                  end;
                finally
                  Track.free;
                end;
                dt := dt + (dt_last - dt_first);
                GB_Progress(2,0,Track_num);
              end;
            end;
          end;
        end;
        TrkForm_ed_TripLenght.Text := FormateFloat(TripLen);
        TrkForm_ed_TripTime.Text   := IntToStr(trunc(dt)) + 'd  ' + FormatDateTime('hh:mm:ss', dt);
        if dt <> 0 then TrkForm_ed_TripSpeed.Text := FormateFloat(TripLen /(dt * 24))
                   else TrkForm_ed_TripSpeed.Text := '0.0';
        if (alt_min = 9999) then alt_min := 0;
        TrkForm_ed_TripAltMin.Text := IntToStr(trunc(alt_min));
        TrkForm_ed_TripAltMax.Text := IntToStr(trunc(alt_max));
      finally
        Track_db.free;
      end;
      GB_Clean;
      GB_Progress(1,0,m);
      form_TrkInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - download databze _adrake.db3 z AD}
procedure form_db_Trk_Dwn(Sender: TObject);
begin
  GB_Progress(1,10,100);
  form_TrkInh(False);
  db_AD_Dwn;                                      // nacte databazi _adrake.db3 z PDA
  form_TrkRead(TrkForm_btn_Refresh);              // nacte aktualizovany seznam stop
  GB_Progress(1,100,100);
  form_TrkInh(True);
  GB_Clean;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - upload databze _adrake.db3 do AD}
procedure form_db_Trk_Upl(Sender: TObject);
begin
  GB_Progress(1,10,100);
  form_TrkInh(False);
  db_AD_Upl;                                      // zapise databazi _adrake.db3 do PDA
  GB_Progress(1,100,100);
  form_TrkInh(True);
  GB_Clean;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Zobrazi okno Tracks}
procedure form_ShowTrkForm(Sender: TObject);
begin
  form_TrkRead(TrkForm_btn_Refresh);    //nacte aktualizovany seznam stop
  GB_Caption(_('Tracks manager showed'));
  TrkForm.ShowModal;
end;
{Zobrazi okno TrkDivide}
procedure form_ShowTrkDivideForm(Sender: TObject);
begin
  if (TrkForm_lb_Track.SelCount > 0) then begin     // pokud je vybrana nejaka stopa
    ReadIniTrkForm('DEFAULT');
    if TrkDivideForm.ShowModal = 1 then begin
      TrkDivide;
      SaveIniTrkForm('DEFAULT');
    end;
  end;
end;
